@@ -130,7 +130,7 @@ class CrashDatabase(apport.crashdb.CrashDatabase):
             return self.__launchpad
 
         if Launchpad is None:
-            sys.stderr.write('ERROR: The launchpadlib Python module is not installed. This functionality is not available.\n')
+            sys.stderr.write('ERROR: The launchpadlib Python %s module is not installed. This functionality is not available.\n' % sys.version[0])
             sys.exit(1)
 
         if self.options.get('launchpad_instance'):
@@ -819,8 +819,12 @@ in a dependent package.' % master,
                 if task.target.resource_type_link.endswith('#distribution'):
                     task.target = self.lp_distro.getSourcePackage(
                         name=report['SourcePackage'])
-                    task.lp_save()
-                    bug = self.launchpad.bugs[id]
+                    try:
+                        task.lp_save()
+                        bug = self.launchpad.bugs[id]
+                    except HTTPError:
+                        # might fail if there is already another Ubuntu package task
+                        pass
                     break
 
         if 'need-duplicate-check' in bug.tags:
